/* get var. pipelen from database */
/* with different offset for the superlayers, but therefor without check_event etc */

/*  CHANGES:
 *
 *  05 feb 2001 (wg): switch off reporting of feds by default 
 *                    parameter 'fed statistics period'
 *
 */

#include <stdio.h>  
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "aframelib.h"
#include "iol.h"
#include "grr.h"
#include "etb.h"

#include "itrleda.h"
#include "ITR_silraw.h"

/* IOL definitions */
ITR_Silraw_Iol_t ITR_silraw_iol={
  xcvsMODULE_ITR_silraw,
  IOL_NVERBS(ITR_Silraw_Iol_t,0),
  {"version"                , xcvsVERSION_ITR_silraw, IOL_TYPE_STRING},
  {"rawdata type"           , 0,         " ",     IOL_TYPE_INT},
  {"datavalid jitter"       , 0,         " ",     IOL_TYPE_INT},
  {"datavalid jitter offset", 0,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl1"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl2"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl3"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl4"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl5"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl6"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl7"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl8"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl9"   , 6,         " ",     IOL_TYPE_INT},
  {"datavalid offset sl10"  , 6,         " ",     IOL_TYPE_INT},
  {"minadcdiff"             , 10,        " ",     IOL_TYPE_INT},
  {"dump header"            , 0,         " ",     IOL_TYPE_INT},
  {"dump chips"             , 0,         " ",     IOL_TYPE_INT},
  {"dump feds"              , 0,         " ",     IOL_TYPE_INT},
  {"dump sparse"            , 0,         " ",     IOL_TYPE_INT},
  {"debug mode"             , 0,         " ",     IOL_TYPE_INT},
  {"get faulty feds"        , 0,         " ",     IOL_TYPE_INT},
  {"fed statistics period"  , 0,         " ",     IOL_TYPE_INT},
  {"bad chip list"          , "none",             IOL_TYPE_STRING},
  {"cell check"             , 0,         " ",     IOL_TYPE_INT},
  {"cell check output"      , 0,         " ",     IOL_TYPE_INT},
  {"event check"            , 0,         " ",     IOL_TYPE_INT},
  {"repair mode"            , 0,         " ",     IOL_TYPE_INT},
};/*ATTENTION WITH "GET FAULTY FEDs"!!!!!!!!!!!!!!*/							    

/* event data */
static ITR_Silraw_Event_t *silraw_ptr = 0;

static int paraminit_done    = 0; /* Flag that param_init was done */
static int startprogram_done = 0; /* Flag that startprogram was done */

/* variables for event checks */
extern int ITR_silraw_evno;           ;
extern int ITR_silraw_evfail_tot;
extern int ITR_silraw_linkfail_tot;
extern int ITR_silraw_cellfail_tot_short;
extern int ITR_silraw_cellfail_tot_long;
extern int ITR_silraw_chipfail[DIM_NCHIP];
extern int ITR_silraw_chipid[DIM_NCHIP];


/* ------------------------------------------------------------------------
 *
 *  Initialize silraw.
 *
 *  To be called before everything else.
 *  Done is the IOL initialization.
 *
 * ------------------------------------------------------------------------ */
int ITR_silraw_param_init()
{
  char *routine = "ITR_silraw_param_init";

  /* Check if we haven't called it yet */
  if ( paraminit_done ) {
    aframe_mess(routine,GRR_SWARNING,
		"ITR_silraw_param_init already called, skipping...");
    return GRR_OK;
  }
  paraminit_done = 1;

  /* tell the user that its alive */
  aframe_mess(routine,GRR_SINFO,
	      "ITR_silraw version %s started.",xcvsVERSION_ITR_silraw);

  /* initilize iol parameters of silraw package */
  iol_declare_structure( (IOL_GENERIC *) &ITR_silraw_iol );

  return GRR_OK;
}


/* ------------------------------------------------------------------------
 *
 *  StartProgram actions.
 *
 *  The ITR_Silraw_Event_t structure is declared to DATPAK.
 *
 * ------------------------------------------------------------------------ */
int ITR_silraw_startprogram()
{
  int   rc;
  int   ichip;

  static ITR_Silraw_Event_t silraw;

  char *routine = "ITR_silraw_startprogram";
  
  /* Check if we haven't called it yet */
  if ( startprogram_done ) {
    aframe_mess(routine,GRR_SINFO,"ITR_silraw_startprogram already called, skipping...");
    return GRR_OK;
  }
  startprogram_done = 1;

  /* declare ITR_Silraw_Event_t to the rest of the world */
  silraw_ptr = &silraw;

  /* fill bad chip list (if requested) */
  rc = ITR_silraw_read_bad_chip_list();
  if ( rc != GRR_OK ) 
    return rc;

  
 
  /* initialize cell check parameters */
  if ( ITR_silraw_iol.cell_check.value ) {
    ITR_silraw_evno         = 1;
    ITR_silraw_evfail_tot   = 0;
    ITR_silraw_linkfail_tot = 0;
    ITR_silraw_cellfail_tot_short = 0;
    ITR_silraw_cellfail_tot_long  = 0;
    for ( ichip=0; ichip<DIM_NCHIP; ichip++ ) {
      ITR_silraw_chipfail[ichip] = 0;
      ITR_silraw_chipid[ichip]   = 0;
    }
  }    

  return GRR_OK;
}

/* ------------------------------------------------------------------------
 *
 *  silraw event function
 *
 *  fills header, do some checks
 *
 * ------------------------------------------------------------------------ */
int ITR_silraw_event( char *rawdata, int nfed )
{
  int rc;

  EVENT_HDR       *event_hdr;
  EVT_DATA_RECORD evt_data;
  ETB_fed         etb_fed;
  int             i, nfadc, n, nchip;

  ITR_Silraw_Event_t *silraw;

  char *routine = "ITR_silraw_event";
 
  /* get ITR_silraw event */
  silraw = ITR_silraw_fetch_event();
  if ( !silraw )
    return GRR_FISHY;

  /* Increase event # by one - may be overwritten later! */
  silraw->header.minor++;

  /* get current event time */
  silraw->header.evtime = time( NULL );

  /* Check input data type */
  if ( ITR_silraw_iol.rawdata_type.value == 0 )
    rc = ITR_silraw_get_event( (EVENT_HDR*) rawdata );
  else if ( ITR_silraw_iol.rawdata_type.value == 1 )
    rc = ITR_silraw_get_slb( (SLB_HDR*) rawdata );
  else if ( ITR_silraw_iol.rawdata_type.value == 2 && nfed > 0 )
    rc = ITR_silraw_get_fed( (FED_HDR*) rawdata, nfed );
  else if ( ITR_silraw_iol.rawdata_type.value == 3 && nfed > 0 )
    rc = ITR_silraw_get_slbsub( (SLB_SUBHDR*) rawdata, nfed );
  else if ( ITR_silraw_iol.rawdata_type.value == 4) {
    event_hdr = (EVENT_HDR*) rawdata;
    rc = ITR_silraw_get_event(event_hdr);
    if ( rc != GRR_OK )
      return GRR_FISHY;
    /*---------------------------------------------------------------*/
    /*  extract: run number
                 event number
                 cell number
                 event time
                 event type 
     */

    rc = ETB_get_daqs( event_hdr, 1, &evt_data, &n );
    if ( rc != ETB_ok && rc != ETB_out_of_memory ) {
      aframe_mess(routine,GRR_SERROR,
		  "No event data in DAQ SLB block found!");
      return GRR_FISHY;
    }

    
    silraw->header.major  = evt_data.data.run_number;
    silraw->header.evtime = evt_data.data.event_time;  
    silraw->header.minor  = ESP_SWAP_INT(event_hdr->sltevt);
    rc = ETB_get_feds( event_hdr, COMP_ID_ITR, 1, &etb_fed, &nfadc ); 
    if ( ( rc != ETB_ok && rc != ETB_out_of_memory ) || nfadc <=0 ) {
      aframe_mess(routine,GRR_SERROR,"No FED data for ITR found!");
      return GRR_FISHY;
    }
    silraw->header.cellshort = etb_fed.hdr.bxid; /*VB: ??????????????????????*/
    silraw->header.celllong  = etb_fed.hdr.bxid;
    rc = GRR_OK;
	    
  }
  else {
    aframe_mess(routine,GRR_SERROR,
		"No valid raw data format given (datatype=%d,nfed=%d)!",
		ITR_silraw_iol.rawdata_type.value, nfed );
    return GRR_FISHY;
  }
  if ( rc != GRR_OK )
    return rc;

  /*--------------------------------------------*/
  /*VB*/

  /*fill the variable pipelen from database*/  
  if( (rc = ITR_silraw_get_nchip (&nchip)) != GRR_OK) return rc;

  if(nchip==0 && silraw->nsparse == 0) 
    return GRR_FISHY;

  for (i = 0; i < nchip; i++) {
    LedaTableRow(ITR_Chip) *ledachip = ITR_Chip->get->key (silraw->chip[i].chipid);
    
    if (ledachip)
      silraw->chip[i].pipelen = ledachip->pipelen;
  }

  /*  for(chipIt.begin();chipIt.valid();chipIt.next()){
      LedaTableRow(ITR_Chip) *ledachip = chipIt.get(); 
      for (i=0;i<nchip;i++){ 
      if(silraw->chip[i].chipid == ledachip->id){ 
      silraw->chip[i].pipelen  = ledachip->pipelen; 
      if(ITR_silraw_iol.debug_mode.value && ITR_silraw_evno%100==0) 
      aframe_mess(routine,GRR_SINFO,"Chip %5d Pipeline %d", 
      silraw->chip[i].chipid,silraw->chip[i].pipelen); 
      } 
      } 
      }
  */



  /* perform event checks, if requested */
  if ( ITR_silraw_iol.event_check.value ) {
    rc = ITR_silraw_check_event();
    if ( rc != GRR_OK ) return rc; 
  } 
  
  if ( ITR_silraw_iol.cell_check.value ) { 
    rc = ITR_silraw_check_cell_consistency(); 
    if ( rc != GRR_OK ) return rc;  
  } 

  /* dump output if requested */
  ITR_silraw_dump_event();

  return GRR_OK;
}


/* Print out end-run statistics */
int ITR_silraw_endrun()
{
  int                       ichip, chipfail_count;
  static ITR_Silraw_Event_t *myevent;

  char *routine = "ITR_silraw_endrun";

  /* print out cell check statistics */
  if ( ITR_silraw_iol.cell_check.value ) {
    aframe_mess(routine,GRR_SINFO,
		"----------------------------------------------");
    aframe_mess(routine,GRR_SINFO,
		"Cell Check Statistics: %8d events", ITR_silraw_evno );
    aframe_mess(routine,GRR_SINFO,
		"                       %8d asynchronous links",
		ITR_silraw_linkfail_tot );
    aframe_mess(routine,GRR_SINFO,
		"                       %8d events with asynch. links",
		ITR_silraw_evfail_tot );
    aframe_mess(routine,GRR_SINFO,
		"                       %8d short cell number inconsistencies",
		ITR_silraw_cellfail_tot_short );
    aframe_mess(routine,GRR_SINFO,
		"                       %8d long cell number inconsistencies",
		ITR_silraw_cellfail_tot_long );

    /* fetch ITR_Silraw_Event_t from datpak */
    myevent = ITR_silraw_fetch_event();
    if ( myevent == NULL ) {
      aframe_mess(routine,GRR_SERROR,
		  "ITR_Silraw_Event_t could not be fetched!");
      return GRR_FISHY;
    }

    /* chip statistics */
    aframe_mess(routine,GRR_SINFO,
		"Chip statistics:  Chip-id   Wrong cell# count since evt 10");
    chipfail_count = 0;
    for ( ichip=0; ichip<2*myevent->nfed; ichip++ )
      if ( ITR_silraw_chipfail[ichip] ) {
	aframe_mess(routine,GRR_SINFO, "                     %4x     %8d",
		    ITR_silraw_chipid[ichip], ITR_silraw_chipfail[ichip] );
	chipfail_count++;
      }
    aframe_mess(routine,GRR_SINFO,
		"Total number of chips : %8d", 2*myevent->nfed );
    aframe_mess(routine,GRR_SINFO,
		"Number of failed chips: %8d", chipfail_count );
    
  }

  aframe_mess(routine,GRR_SINFO,
	      "----------------------------------------------");

  return GRR_OK;
}

/*--------------------------------------------------------------*/
/* ITR_silraw access routine for the rest of the world */
ITR_Silraw_Event_t* ITR_silraw_fetch_event()
{
  char *routine = "ITR_silraw_fetch_event";

  if ( !silraw_ptr )
    aframe_mess(routine,GRR_SINFO,"No ITR_Silraw_Event_t found");

  return silraw_ptr;
}
