#ifndef IOL_H
#define IOL_H

#ifdef __cplusplus
extern "C" {
#endif

#include "CVStag.iol"
typedef char IOL_DESCRIPTION;

/*
 * iol.h needs FILE struct, so this include is necessary.
 * T. Glebe, 25.02.1998
 */
#include <stdio.h>

/*
 * Replaced AFRAME-Signals by their numerical values.
 * You don't need to include aframe.h before iol.h any more!!
 * T. Glebe, 25.02.1998
 */
#define IOL_OK         0
#define IOL_NOTOK     -1

/*
 * Option flags for init
 * ---------------------
 */
#define IOL_RPM_FLAG  0x1



#define IOL_NLEN 32       /* Max Length of string describing verb name */
#define IOL_TLEN 8        /* Max Length of string describing type      */
#define IOL_CLEN 32       /* Character value length                    */
#define IOL_MAX_ALLEN 132 /* Max lenght of a line to be scanned        */

typedef char  IOL_MNAME;
typedef int   IOL_SIZE;           
typedef char  IOL_VNAME;
typedef char  IOL_TYPE;

#define IOL_MLEN 32-sizeof(IOL_SIZE)      /* Max Length of string describing module    */

/* The following types of values are defined */

typedef char   IOL_VSTRING; /* string value */
typedef int    IOL_VINT;    /* int  value   */
typedef int    IOL_VFLAG;    /* logical flag  value  (can be 0 or 1)*/
typedef float  IOL_VFLOAT;  /* float value  */
typedef double IOL_VDOUBLE; /* double value */
typedef int    IOL_VHEX;    /* hex value    */

/* Padding to equalize the memory for the various types */

typedef char   IOL_DUMMY;   /* dummy padding needed for identical memory */

#define IOL_TYPE_STRING "S"
#define IOL_TYPE_FLAG   "L"
#define IOL_TYPE_INT    "I"
#define IOL_TYPE_FLOAT  "F"
#define IOL_TYPE_DOUBLE "D"
#define IOL_TYPE_HEX    "H"

typedef struct iol_generic{
 IOL_MNAME   module[IOL_MLEN]    ;       /* The module name must be first      */
 IOL_SIZE    size            ;       /* The number of verbs must be second */
}  IOL_GENERIC;

/* offset from beginning of structure to the user preferences              */

#define IOL_OFFSET ( ( (IOL_MLEN)*(sizeof(IOL_MNAME)) ) + \
                      ( sizeof(IOL_SIZE) )         )

/* offset from the beginning of the structure to the verb type description */

#define IOL_TOFF   ( ( (IOL_NLEN)*( sizeof(IOL_VNAME) ) ) + \
                          ( (IOL_CLEN)*( sizeof(IOL_VSTRING) ) ) )   

/* offset from the beginning of the structure to the verb itself           */
#define IOL_VOFF     ( (IOL_NLEN)*( sizeof(IOL_VNAME) ) )   

/* Length of dummy array paddings */

#define IOL_DLENL ((IOL_CLEN)-sizeof(IOL_VFLAG))    /* for logical flag  */
#define IOL_DLENI ((IOL_CLEN)-sizeof(IOL_VINT))     /* for int   */
#define IOL_DLENF ((IOL_CLEN)-sizeof(IOL_VFLOAT))   /* for float */
#define IOL_DLEND ((IOL_CLEN)-sizeof(IOL_VDOUBLE))  /* for double */
#define IOL_DLENH ((IOL_CLEN)-sizeof(IOL_VHEX))     /* for hex   */

#define IOL_NVERBS(Y,X)   ( ( sizeof(Y)           ) - \
                          ( (IOL_MLEN)*( sizeof(IOL_MNAME) ) ) - \
                          ( sizeof(IOL_SIZE)             ) - \
                          ( ( X )))/(sizeof(IOL_STRING))  

#define IOL_BYTESIZE(NVERBS)  ( (IOL_MLEN)*( sizeof(IOL_MNAME)  ) ) + \
                          (              sizeof(IOL_SIZE )    ) + \
                          ( ( NVERBS )*( sizeof(IOL_STRING) ) )  

/* Defintions of the structures for the various cases of
   verb types */

typedef struct IOL_string{
     IOL_VNAME name[IOL_NLEN] ;
     IOL_VSTRING value[IOL_CLEN];
     IOL_TYPE  type[IOL_TLEN] ;
} IOL_STRING      ;

typedef struct IOL_int{
     IOL_VNAME name[IOL_NLEN];
     IOL_VINT  value;
     IOL_DUMMY dummy[IOL_DLENI];
     IOL_TYPE  type[IOL_TLEN];
} IOL_INT;

typedef struct IOL_flag{
     IOL_VNAME name[IOL_NLEN];
     IOL_VFLAG  value;
     IOL_DUMMY dummy[IOL_DLENL];
     IOL_TYPE  type[IOL_TLEN];
} IOL_FLAG;

typedef struct IOL_float{
     IOL_VNAME  name[IOL_NLEN];
     IOL_VFLOAT value     ;
     IOL_DUMMY  dummy[IOL_DLENF];
     IOL_TYPE   type[IOL_TLEN];
} IOL_FLOAT       ;

typedef struct IOL_double{
     IOL_VNAME   name[IOL_NLEN];
     IOL_VDOUBLE value     ;
     IOL_DUMMY   dummy[IOL_DLEND];
     IOL_TYPE    type[IOL_TLEN];
} IOL_DOUBLE       ;

typedef struct IOL_hex{
     IOL_VNAME name[IOL_NLEN];
     IOL_VHEX  value;
     IOL_DUMMY dummy[IOL_DLENH];
     IOL_TYPE  type[IOL_TLEN];
} IOL_HEX;

/* Below are sets of inline functions for accessing information
   about the structure.  The access can be done using
   a pointer to the structure (such as XMOD) and the ordinal
   location of the verb/value structure counting from 0 (=XVERB). */

/* Access to the type of the value stored */
   
#define IOL_GET_TYPE(XMOD,XVERB) (char*)(XMOD.module+(XVERB)*(sizeof(IOL_STRING))\
 + (IOL_OFFSET) + (IOL_TOFF))

/* Access to the verb string describing the value */
#define IOL_GET_VERB(XMOD,XVERB) (char*)(XMOD.module+(XVERB)*(sizeof(IOL_STRING))\
 + (IOL_OFFSET))

#define IOL_LOC_PTR_VOID(XMOD,XVERB) (((XMOD).module)+(XVERB)*(sizeof(IOL_STRING))\
 + (IOL_OFFSET) + (IOL_VOFF))

/* Access to a string value */
#define IOL_LOC_VAL_S(XMOD,XVERB) ((IOL_VSTRING*)(IOL_LOC_PTR_VOID(XMOD,XVERB)))

/* Access to an integer value */
#define IOL_LOC_VAL_I(XMOD,XVERB) (*((IOL_VINT*)(((XMOD).module)+(XVERB)*(sizeof(IOL_STRING))\
 + (IOL_OFFSET) + (IOL_VOFF))))

/* Access to an logical flag value */
#define IOL_LOC_VAL_L(XMOD,XVERB) (*((IOL_VFLAG*)(XMOD.module+(XVERB)*(sizeof(IOL_STRING))\
 + (IOL_OFFSET) + (IOL_VOFF))))

/* Access to a float value */
#define IOL_LOC_VAL_F(XMOD,XVERB) (*((IOL_VFLOAT*)(XMOD.module+(XVERB)*(sizeof(IOL_STRING))\
 + (IOL_OFFSET) + (IOL_VOFF))))

/* Access to a double value */
#define IOL_LOC_VAL_D(XMOD,XVERB) (*((IOL_VDOUBLE*)(XMOD.module+(XVERB)*(sizeof(IOL_STRING))\
 + (IOL_OFFSET) + (IOL_VOFF))))

/* Access to a hex value value */
#define IOL_LOC_VAL_H(XMOD,XVERB) (*((IOL_VHEX*)(XMOD.module+(XVERB)*(sizeof(IOL_STRING))\
 + (IOL_OFFSET) + (IOL_VOFF))))

/* generic access to a value */

#define IOL_LOC_VAL(XMOD,XVERB)  \
(!strcmp(IOL_GET_TYPE(XMOD,XVERB),"S")) ? IOL_LOC_VAL_S(XMOD,XVERB) : \
(!strcmp(IOL_GET_TYPE(XMOD,XVERB),"L")) ? IOL_LOC_VAL_L(XMOD,XVERB) : \
(!strcmp(IOL_GET_TYPE(XMOD,XVERB),"I")) ? IOL_LOC_VAL_I(XMOD,XVERB) : \
(!strcmp(IOL_GET_TYPE(XMOD,XVERB),"F")) ? IOL_LOC_VAL_F(XMOD,XVERB) : \
(!strcmp(IOL_GET_TYPE(XMOD,XVERB),"D")) ? IOL_LOC_VAL_D(XMOD,XVERB) : \
(!strcmp(IOL_GET_TYPE(XMOD,XVERB),"H")) ? IOL_LOC_VAL_H(XMOD,XVERB) : IOL_ERR

/* print descriptor for a value */

#define IOL_GET_PRINT_DESC(XMOD,XVERB) \
 (!strcmp((IOL_GET_TYPE(XMOD,XVERB)),"S"))?"%s" :       \
 (!strcmp((IOL_GET_TYPE(XMOD,XVERB)),"I"))?"%d" :       \
 (!strcmp((IOL_GET_TYPE(XMOD,XVERB)),"L"))?"%d" :       \
 (!strcmp((IOL_GET_TYPE(XMOD,XVERB)),"F"))?"%f" :       \
 (!strcmp((IOL_GET_TYPE(XMOD,XVERB)),"D"))?"%f" :       \
 (!strcmp((IOL_GET_TYPE(XMOD,XVERB)),"H"))?"%x" : "%s" 

/* print value */
#define IOL_FPRINT_VALUE(XFP,XMOD,XIV) \
(!strcmp((IOL_GET_TYPE(XMOD,XIV)),"S")) ?\
     fprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),(IOL_LOC_VAL_S(XMOD,XIV)))\
     :\
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"I")) ?\
     fprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),(IOL_LOC_VAL_I(XMOD,XIV)))\
     :\
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"L")) ?\
     fprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),IOL_LOC_VAL_L(XMOD,XIV))\
     :\
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"F")) ?\
     fprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),IOL_LOC_VAL_F(XMOD,XIV))\
     :\
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"D")) ?\
     fprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),IOL_LOC_VAL_D(XMOD,XIV))\
     :\
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"H")) ?\
     fprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),IOL_LOC_VAL_H(XMOD,XIV))\
     :\
     fprintf(XFP,"IOL: Unknown type")
#define IOL_SPRINT_VALUE(XFP,XMOD,XIV) \
(!strcmp((IOL_GET_TYPE(XMOD,XIV)),"S")) ?\
     sprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),(IOL_LOC_VAL_S(XMOD,XIV))) \
     : \
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"I")) ?\
     sprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),(IOL_LOC_VAL_I(XMOD,XIV)))\
     : \
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"L")) ?\
     sprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),IOL_LOC_VAL_L(XMOD,XIV))\
     :\
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"F")) ?\
     sprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),IOL_LOC_VAL_F(XMOD,XIV))\
     :\
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"D")) ?\
     sprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),IOL_LOC_VAL_D(XMOD,XIV))\
     :\
     (!strcmp((IOL_GET_TYPE(XMOD,XIV)),"H")) ?\
     sprintf(XFP,IOL_GET_PRINT_DESC(XMOD,XIV),IOL_LOC_VAL_H(XMOD,XIV))\
     :\
     sprintf(XFP,"%s","IOL: Unknown type")

#define IOL_PRINT_VALUE(XMOD,XIV) IOL_FPRINT_VALUE(stdout,XMOD,XIV) \

struct iol_list_struct {
  IOL_GENERIC  *structure_ptr;
  IOL_GENERIC  *default_structure_ptr;
  struct iol_list_struct *next;
};
typedef struct iol_list_struct IOL_LIST_STRUCT;
typedef IOL_LIST_STRUCT *IOL_LIST;


#define IOL_VERSION xcvsVERSION_iol

/* Iol control structure -- iol controls iol! 
   ========================================== */

struct iol_struct_def {
  IOL_MNAME   module[IOL_MLEN];       /* The module name must be first       */
  IOL_SIZE    size            ;       /* The number of verbs must be second */ 
  IOL_STRING  version         ;       /* program version number              */
  IOL_INT     debug_mode      ;       /* debug mode                          */
  IOL_FLAG    match_verb_exact;       /* requires exact verb matching        */
  IOL_FLAG    protect_version ;       /* prevents changes to the version verb*/
  IOL_FLAG    report_aliens   ;       /* report lines that are not matched   */
};
typedef struct iol_struct_def IOL_STRUCT;

#define IOL_IOL_NVERBS IOL_NVERBS(IOL_STRUCT,0)

#define IOL_NAME "iol"


/* Function prototypes 
   =================== */
int iol_param_init        ();
int iol_init              ( const char* dummy1, unsigned int dummy2 );
int iol_declare_structure (IOL_GENERIC *structure_ptr);
int iol_match_module      (char* module_name, IOL_GENERIC **structure_ptr,IOL_GENERIC **default_structure_ptr);
int iol_locate_verb       (char* verb, IOL_GENERIC *structure_ptr, int *verb_location);
int iol_scan_preferences  (char* filename);

int iol_print_to_screen   ();
int iol_print_to_file     (char *file);
int iol_print_values      (FILE *fp);

int iol_dumpval           (FILE *fp, IOL_GENERIC *struct_ptr, IOL_GENERIC *default_structure_ptr, int maxmod_len);
int iol_dumpstruct        (IOL_GENERIC *struct_ptr, int nele);

int iol_load_value        (IOL_GENERIC *struct_ptr, int loc, char *val);
int iol_load_value_i      (IOL_GENERIC *struct_ptr, int loc, int *val); 
int iol_load_value_l      (IOL_GENERIC *struct_ptr, int loc, int *val); 
int iol_load_value_f      (IOL_GENERIC *struct_ptr, int loc, float *val);
int iol_load_value_d      (IOL_GENERIC *struct_ptr, int loc, double *val);
int iol_load_value_h      (IOL_GENERIC *struct_ptr, int loc, int *val); 
int iol_load_value_s      (IOL_GENERIC *struct_ptr, int loc, char *val);

int iol_remove_inline_comment(char *string);
int iol_remove_blanks(char *string);
int iol_sanitize_string(char *string);
int iol_protected_verb_violation(char *verb);
int iol_match_verbs(char *verb, char* reference);
int iol_protection_on(char *verb);
int iol_load_value_from_mod_verb (char* value, 
				  char* module, char *verb, 
				  int *module_matches, int *verb_matches);
int iol_fetch_value_from_mod_verb(void **val,
				  char* module, char *verb, 
				  int *module_matches, int *verb_matches);
int iol_menu(int print_menu);
int iol_dump_module_names(FILE *fp, int *nmodules);
int iol_restore_all_defaults();
int iol_restore_defaults(char *module);
int iol_terminate         ();
int iol_fetch_module(char module[IOL_MLEN],int button);
int iol_load_value_from_mod_verb_line(char* value, char* module,
				      char *verb_line, char* verb,
				      int *module_matches,
				      int *verb_matches);


#define IOL_CHECK_ALLOC(mem_pointer,saved_value)                         \
{ if(mem_pointer==NULL){                                                 \
          printf("iol_save_value: unable to allocate room to save %s\n", \
		 saved_value);                                           \
	  return IOL_NOTOK;                                              \
}}

#ifdef __cplusplus
}
#endif

#endif
