/*----------------------------------------------------------------------------
 *
 *    Author: R. Wanke, 12-Nov-1997
 *
 *    Short Description:
 *    ==================
 *    Converts date and time to unix-time
 *
 *    Formats:  Date: yymmdd  OR yyyymmdd
 *              Time: hhmmss
 *              DST :  0/1 (Daylight Savings Time OFF/ON) 
 *    
 *---------------------------------------------------------------------------*/


/* include files  */
#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <time.h>
#include  <ctype.h>
#include  <math.h>
#include "grr.h"

/*============================================================================
 *
 *             main program
 *
 *==========================================================================*/
int datetime_to_utime(int date, int time, int dst_flag, int * utime )
{

  /* structure from time.h */
  struct tm mytime;

/*--------------------------------------------------------------------------*/

  /* check date & time */
  if ( time < 0 || date <= 0 ) {
    printf("Invalid date %d or time %d, leaving...\n", date, time );
    return GRR_FISHY;
  }
  printf("Date = %d\nTime = %d\n", date, time );

  /* daylight savings time flag */
  mytime.tm_isdst = dst_flag;
  if ( mytime.tm_isdst > 0 )   printf("Daylight Savings Time ON\n"); 
  if ( mytime.tm_isdst == 0 )  printf("Daylight Savings Time OFF\n"); 
  if ( mytime.tm_isdst < 0 )
    printf("Don't know about Daylight Savings Time\n");

  /* fill time in tm-structure*/
  mytime.tm_hour = floor( time / 10000 );
  time -= 10000 * mytime.tm_hour;
  mytime.tm_min = floor( time / 100 );
  time -= 100 * mytime.tm_min;
  mytime.tm_sec = time;

  /* fill date in tm-struct */
  mytime.tm_year = floor( date / 10000 );
  date -= 10000 * mytime.tm_year;
  mytime.tm_mon = floor( date / 100 );
  date -= 100 * mytime.tm_mon;
  mytime.tm_mday = date;
  if ( mytime.tm_year > 1900 )
    mytime.tm_year -= 1900;
  if ( mytime.tm_year < 50 )  /* year 2000 problem! */
    mytime.tm_year += 100;
  mytime.tm_mon--;    /* months count from 0 to 11 */

  /* check items */
  if ( mytime.tm_hour < 0 || mytime.tm_hour > 23 ) {
    printf("Hour %d not in valid range (0 <= hour <= 23)\n", mytime.tm_hour);
    return GRR_FISHY;
  }
  if ( mytime.tm_min < 0 || mytime.tm_min > 59 ) {
    printf("Min %d not in valid range (0 <= min <= 59)\n", mytime.tm_min);
    return GRR_FISHY;
  }
  if ( mytime.tm_sec < 0 || mytime.tm_sec > 61 ) {
    printf("Sec %d not in valid range (0 <= sec <= 61)\n", mytime.tm_sec);
    return GRR_FISHY;
  }
  if ( mytime.tm_mday < 1 || mytime.tm_mday > 31 ) {
    printf("Day %d not in valid range (1 <= day <= 31)\n", mytime.tm_mday);
    return GRR_FISHY;
  }
  if ( mytime.tm_mon < 0 || mytime.tm_mon > 11 ) {
    printf("Mon %d not in valid range (1 <= mon <= 12)\n", mytime.tm_mon+1);
    return GRR_FISHY;
  }
  if ( mytime.tm_year < 50 || mytime.tm_year > 150 ) {
    printf("Year %d not in valid range (50 <= year <= 150)\n", mytime.tm_year);
    return GRR_FISHY;
  }

  /*  printf(" Hour = %d\n", mytime.tm_hour );
      printf(" Min  = %d\n", mytime.tm_min );
      printf(" Sec  = %d\n", mytime.tm_sec );
      printf(" Day  = %d\n", mytime.tm_mday );
      printf(" Mon  = %d\n", mytime.tm_mon );
      printf(" Year = %d\n", mytime.tm_year );
  */

  /* convert to start unix-time */
  *utime = mktime( &mytime );

  printf("Requested Date & Time: %s\n", asctime( &mytime ) );
  printf("This converts to unix-time in sec after the epoch: %d\n\n", *utime);

  return GRR_OK;

}


/* Transform Unix time into non-\n-terminated string */
char *print_date( int utime )
{
  char *chtime;
  int  i;
  /*
  return asctime( localtime( (time_t*) &utime) );
  strcpy( chtime, ctime( (time_t*) &utime ) );
  */

  chtime = ctime( (time_t*) &utime );
  i      = 0;
  while ( !iscntrl(chtime[i]) )
    i++;
  chtime[i] = '\0';

  return chtime;
}
