#include <string.h>
#include <stdlib.h>

#include "iol.h"
#include "aframelib.h"
#include "grr.h"

#include "database/table.h"

#define NUMLINES 300
#define MAXSIZE  64

int iol_scan_preferences_from_db(char* dbase, int major, int minor)
{
  char          *verb;
  int           value_load_flag;
  int           module_matches,verb_matches;
  int           stat;                         /* Overall status of routine*/
  int           aliens_found;                 /* number of unknown lines */
  int           line;
  char          *routine = "iol_scan_preferences_from_db";
  char          **values, **verbs, **modules;
  Tables        db;
  int           tx, numEntries, imaj, imin, rc;
  char          *tablename_default = "ITR/Steering";
  char          server[256], *tablename, *tmp;
 
  values  = malloc (sizeof (char**) * NUMLINES);
  verbs   = malloc (sizeof (char**) * NUMLINES);
  modules = malloc (sizeof (char**) * NUMLINES);

  if (values == 0 || verbs == 0 || modules == 0) {
    aframe_mess(routine, GRR_SERROR, "Couldn't allocate enough memory!");
    if (values)  free (values);
    if (verbs)   free (verbs);
    if (modules) free (modules);
    return (GRR_FISHY);
  }

  /* --wg-- initialize values, verbs and modules to NULL such that realloc can work */
  values[0] = NULL;
  verbs [0] = NULL;
  modules[0]= NULL;


  /* Assume ok! */
  stat = IOL_OK;

  /* Init to no match */
  value_load_flag = 0;
  line = 0;
  verb = malloc( (IOL_NLEN+1)*sizeof(char) ); /* make room for the verb */
  aliens_found = 0;                           /* note if unknown lines were found */

  /* Server and Table can be given to this routine in one string separated with a colon,
     like  "/ITR_Setup:ITR/Steering" */
  /* work on a copy of argument dbase */
  strncpy (server, dbase, 255);
  if ( (tmp = strchr (server, ':')) != NULL) {
    tablename = tmp + 1;
    *tmp = '\0';
  }
  else
    tablename = tablename_default;
  

  aframe_mess (routine, GRR_SINFO, "Requesting %s:%s version (%d, %d)", 
	       server, tablename, major, minor);

  db = OpenTables (server, 0);

  if ( db != NULL ) {
    tx = BookTable  (db, tablename, NULL, NULL);
    if (tx == 0) {
      aframe_mess (routine, GRR_SERROR, "Cannot find table `%s' in server `%s'", 
		   tablename, server);
      return (GRR_FISHY);
    }

    TableStamp      (db, major, minor);
    MatchTable      (db, tx, TABLEle);
    imaj = TableMajor (db);
    imin = TableMinor (db);
    aframe_mess (routine, GRR_SINFO, "Got Database %s:%s version (%d, %d)", 
		 server, tablename, imaj, imin);


    rc = FetchTable (db, tx, TABLEle);
    if (rc == 0) {
      aframe_mess (routine, GRR_SERROR, "Cannot FetchTable()");
      return (GRR_FISHY);
    }

    SkipTableField (db, 1);
    numEntries = GetTableField  (db, TABLEstring, NUMLINES, values);
    
    GetTableField  (db, TABLEstring, NUMLINES, modules);
    GetTableField  (db, TABLEstring, NUMLINES, verbs);

    /* Scan for matches */
    line=0;
    while ( line < numEntries && 
	    line < NUMLINES &&
	    values [line] != NULL ) {

      /* Load the value from the module and verb */
      if ( iol_load_value_from_mod_verb_line(values[line], 
					     modules[line], 
					     verbs[line],
					     verb,
					     &module_matches,
					     &verb_matches) != IOL_OK) {
	
	/* if ( iol_struct.report_aliens.value ) { */
	if ( 0 ) {
	    aframe_mess(routine,GRR_SWARNING,"UNKNOWN LINE");
	    aframe_mess(routine,GRR_SWARNING,"value : >%s<",values[line]);
	    aframe_mess(routine,GRR_SWARNING,"module: >%s< [%s]",modules[line],
			module_matches ? "matched" : "not matched");
	    aframe_mess(routine,GRR_SWARNING,"verb  : >%s< [%s]",verbs[line],
			verb_matches ? "matched" : "not matched");
	}
	aliens_found++;       /* increment the counter of unknown lines */
      }
      line ++;
    }
    CloseTables (db);
  }
  else {
      aframe_mess(routine,GRR_SERROR,"Failed to open database: %s", server);
      stat=IOL_NOTOK;
  }
  free(verb);

  if ( stat == IOL_OK && aliens_found > 0 )
    stat = aliens_found;
  else if ( line == 0 )
    stat = IOL_NOTOK;
  if ( line == 0 ) 
    aframe_mess(routine,GRR_SWARNING,
		"no matches found in database %s", server);

  if (values)  free (values);
  if (verbs)   free (verbs);
  if (modules) free (modules);

  return stat;
}
